# -*- coding: utf-8 -*-

__author__ = """gawel <gawel@afpy.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.ATContentTypes.content.folder import registerATCT, ATFolder, ATFolderSchema
from Products.FlickrAlbum.config import *

import logging
log = logging.getLogger(PROJECTNAME)

FlickrAlbum_Schema = ATFolderSchema.copy() + Schema((

    StringField(
        name='user_id',
        schemata='sets',
        widget=StringWidget(
            macro='flickr_userid_widget',
            label='User id',
            label_msgid='label_user_id',
            i18n_domain='FlickrAlbum',
        )
    ),
    
    LinesField(
        name='sets',
        schemata='sets',
        vocabulary = 'getSetsForDisplay',
        widget=MultiSelectionWidget(
            format='select',
            label='Sets',
            label_msgid='label_set',
            i18n_domain='FlickrAlbum',
        )
    ),

    LinesField(
        name='tags',
        schemata='search',
        widget=LinesWidget(
            label='Tags',
            label_msgid='label_tags',
            i18n_domain='FlickrAlbum',
        )
    ),

    StringField(
        name='tag_mode',
        schemata='search',
        default='any',
        vocabulary = DisplayList( (('any','Match any tag'), ('all','Match all tags')) ),
        widget=SelectionWidget(
            label='Tag mode',
            label_msgid='label_tag_mode',
            i18n_domain='FlickrAlbum',
        )
    ),

    StringField(
        name='sort',
        schemata='search',
        vocabulary = DisplayList( (('date-posted-asc','Date posted asc'),
                                   ('date-posted-desc','Date posted desc'),
                                   ('date-taken-asc','Date taken asc'),
                                   ('date-taken-desc','Date taken desc'),
                                   ('interestingness-desc','Interestingness desc'),
                                   ('interestingness-asc','Interestingness asc')) ),
        widget=SelectionWidget(
            label='Sort by',
            label_msgid='label_sort',
            i18n_domain='FlickrAlbum',
        )
    ),

),
)

class FlickrAlbum(ATFolder):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = ATFolder.__implements__

    # This name appears in the 'add' box
    archetype_name = 'FlickrAlbum'

    meta_type = 'FlickrAlbum'
    portal_type = 'FlickrAlbum'
    allowed_content_types = ['Image']
    filter_content_types = 1
    global_allow = 1
    allow_discussion = False
    #content_icon = 'FlickrAlbum.gif'
    immediate_view = 'flickr_album_view'
    default_view = 'flickr_album_view'
    suppl_views = ('flickr_album_view',)
    typeDescription = "FlickrAlbum"
    typeDescMsgId = 'description_edit_flickralbum'

    _at_rename_after_creation = True

    schema = FlickrAlbum_Schema
    
    actions = ({
        'id'          : 'view',
        'name'        : 'View',
        'action'      : 'string:${object_url}',
        'permissions' : ('View',)
         },
        {'id': 'update-contents',
         'name': 'Update contents',
         'action': 'string:$object_url/updateContents',
         'permissions': ('Modify portal content,'),
         'category': 'object_buttons'
        },
    )
    # Methods

    def getSetsForDisplay(self,userid=None):
        from Products.FlickrAlbum.flickr_apy import getUserId, getSetsList
        userid = self.getUser_id()
        userid = getUserId(userid)
        if userid:
            sets = getSetsList(userid)
            sets = [(s.getId(),s.Title()) for s in sets]
            return DisplayList( [('','<no set>')] + sets )
        return DisplayList((('','<no sets>'),))


registerATCT(FlickrAlbum, PROJECTNAME)



