import random
from Products.Archetypes.Field import escape
from Products.CMFCore.utils import getToolByName
from zope.app.annotation.attribute import AttributeAnnotations
from zope.interface import implements

from Acquisition import aq_inner, aq_parent
from interfaces import ISlideShow, IScalable, IFlickrImageAble
from config import *

import logging
log = logging.getLogger(PROJECTNAME)

sizes_mapping = {
    'full':'large',
    'large':'large',
    'preview':'medium',
    'mini':'small',
    'thumb':'thumbnail',
    'listing':'square',
    'tile':'square',
    'icon':'square',
  }
  

class FlickrImage(object):

    implements(IScalable)

    def __init__(self,context):
        self.context = context
        self.annotations = AttributeAnnotations(self.context)
        
    def getScaleIndex(self,scale):
        scales = self.annotations.get('flickr_scales',[])

        # sometimes we have the original, sometimes not..
        # so we have to check the 'full' scale
        if scale in ('full','large') and 'original' in scales:
            return scales.index('original')

        new_scale = sizes_mapping.get(scale,scale)
        try:
            index = scales.index(new_scale)
        except ValueError:
            log.info('trying to map %s scale failed for %s' % (scale,
                                                self.context.getId()))
            index = 0
        return index

    def getWidth(self,scale='full'):
        """
        """
        index = self.getScaleIndex(scale)
        widths = list(self.annotations.get('flickr_widths',[]))
        try:
            return widths[index]
        except IndexError:
            return ''

    def getHeight(self,scale='full'):
        """
        """
        index = self.getScaleIndex(scale)
        heights = list(self.annotations.get('flickr_heights',[]))
        try:
            return heights[index]
        except IndexError:
            return ''

    def getScaleUrl(self,scale='full'):
        """
        """
        index = self.getScaleIndex(scale)
        urls = list(self.annotations.get('flickr_urls',[]))
        try:
            return urls[index]
        except IndexError:
            return ''

    def getAvailableSizes(self):
        """
        """
        return AVAILABLE_SIZES

    def tag(self, scale='full', height=None, width=None, alt=None,
            css_class=None, title=None, **kwargs):
        """
        image tag
        """
        context_title = self.context.Title()
        values = {'src' : self.getScaleUrl(scale),
                  'alt' : escape(alt and alt or context_title, 1),
                  'title' : escape(title and title or context_title, 1),
                  'height' : self.getHeight(scale),
                  'width' : self.getWidth(scale),
                 }

        result = '<img src="%(src)s" alt="%(alt)s" title="%(title)s" '\
                 'height="%(height)s" width="%(width)s"' % values

        if css_class is not None:
            result = '%s class="%s"' % (result, css_class)

        for key, value in kwargs.items():
            if value:
                result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result


class SlideShow(object):
    """
    adapter to provide some js stuff
    """
    implements(ISlideShow)
    
    def __init__(self,context):
        self.context = context

    def getContents(self):
        context = self.context
        if context.isPrincipiaFolderish:
            container = aq_inner(context)
            b_start = 0
        else:
            container = aq_parent(aq_inner(context))
            b_start = context.getObjPositionInParent()

        batch = container.getFolderContents()

        sizes = AVAILABLE_SIZES

        # js structure
        Context = "function getContext() { context = new BaseContent('%s'"+",'%s'"*8+"); }\n"
        BaseContent = "    photo = new BaseContent('%s'"+",'%s'"*8+");\n"
        Photo = "    ATPhoto(photo,%s,'%s');\n"
        PhotoSizes = "    photo.setSize('%s',%s,%s,'%s');\n"

        
        out = [Context % (context.absolute_url(), 'id',
                          'title', 'description',
                          context.meta_type,
                          context.portal_type,
                          context.isPrincipiaFolderish,
                          'published','n/a')]

        out.append('''
var b_start = %s;
var contents_loaded = false;
function getSlideShowContents(show_folderish) {
    if (contents_loaded) return;
    var photo;
''' % (b_start,))

        # create a js object for each batch item
        i = 0
        for p in batch:
            # FIXME sometimes an object is not correctly removed from catalog..
            try:
                photo = p.getObject()
            except AttributeError:
                continue
            if IFlickrImageAble.providedBy(photo):
                adapted = IScalable(photo)
            else:
                continue

            # init BaseContent
            title = p.Title or p.getId
            url = p.getURL() + '/view'
            out.append(BaseContent % (url, p.getId,
                                title.replace("'","\\\'"),
                                p.Description.replace("'","\\\'"),
                                p.meta_type,
                                p.Type,
                                p.is_folderish == True and 'true' or 'false',
                                p.review_state,
                                p.getObjSize or 'n/a'))
            # init Photo object
            out.append(Photo % (i,'image/jpeg'))
            # adding scales
            out.extend([ PhotoSizes % (scale,
                                       adapted.getWidth(scale=scale),
                                       adapted.getHeight(scale=scale),
                                       adapted.getScaleUrl(scale=scale)) for scale in sizes])
            # add to js batch
            out.append('    if ( !photo.is_folderish || (show_folderish && photo.is_folderish) ) { Batch.additem(photo); };\n')

            i = i + 1
            
        out.append('    contents_loaded = true;\n}\n')

        out.append('''
function beforeSlideShowInit() {
    Batch.gotoPage(%s);
}''' % b_start)

        return ''.join(out)

