from Products.Five import BrowserView
from zope.app.annotation.attribute import AttributeAnnotations
from interfaces import ISlideShow
from config import PROJECTNAME
from flickr_apy import getUserId, getSetsList, getSet, search

import logging
log = logging.getLogger(PROJECTNAME)


class SlideShowView(BrowserView):

    def getContents(self):
        ''' return js stuff for slideshow '''
        self.request.RESPONSE.setHeader('Content-Type','text/plain')
        annotations = AttributeAnnotations(self.context)
        javascript = annotations.get('slideshow_contents','')
        if not javascript:
            adapted = ISlideShow(self.context)
            javascript = adapted.getContents()
            annotations['slideshow_contents'] = javascript
        return javascript

class FlicrAjaxView(BrowserView):

    def getSetsList(self,userid=None):
        """
        return set list for a user
        """
        self.request.response.setHeader('Content-Type','text/plain')
        userid = self.request.get(userid,userid)
        userid = getUserId(userid)
        if userid:
            sets = getSetsList(userid)
            sets = ['%s||%s' % (s.getId(),s.Title()) for s in sets]
            return '\n'.join(sets)
        return ''

    def checkUserId(self,userid=None):
        """
        return flickr user id if exist
        """
        userid = self.request.get(userid,userid)
        self.request.response.setHeader('Content-Type','text/plain')
        userid = getUserId(userid)
        return userid

class FlickrAlbumView(BrowserView):

    def updateContents(self):
        """
        """
        kwargs = {}
        photos = []
        user_id = self.context.getUser_id()
        user_id = getUserId(user_id)
        if user_id:
            kwargs['user_id'] = user_id
        sets = self.context.getSets()
        if sets and user_id:
            available_sets = getSetsList(user_id)
            for setid in sets:
                for set in available_sets:
                    if set.getId() == setid:
                        set = getSet(set.node)
                        if len(set):
                            photos.extend( set )
        else:
            tags = ','.join(self.context.getTags())
            if tags:
                kwargs['tags'] = tags
            tag_mode = self.context.getTag_mode()
            if tag_mode:
                kwargs['tag_mode'] = tag_mode
            if kwargs:
                results = search(**kwargs)
                if len(results):
                    photos = results.objectValues()

        for id in self.context.objectIds():
            self.context.manage_delObjects(ids=[id])
        for photo in photos:
            sizes = photo.getAvailableSizes()
            urls = []
            heights = []
            widths = []
            
            for size in sizes:
                urls.append( photo.getURL(scale = size) )
                widths.append( photo.getWidth(scale = size) )
                heights.append( photo.getHeight(scale = size) )
                
            if urls and widths and heights:
                if photo.getId() not in self.context.objectIds():
                    self.context.invokeFactory(type_name='Image',id=photo.getId())
                value = {'flickr_scales':sizes,
                         'flickr_widths':widths,
                         'flickr_heights':heights,
                         'flickr_urls':urls}
                obj = self.context[photo.getId()]
                obj.setTitle( photo.Title() )
                obj.setDescription( photo.Description() )
                obj.setExcludeFromNav(True)
                annotations = AttributeAnnotations(obj)
                try:
                    del annotations['slideshow_contents']
                except KeyError:
                    pass
                for k,v in value.items():
                    annotations[k] = v
                if not obj.hasProperty('layout'):
                    obj.manage_addProperty(id='layout',
                            value='flickr_photo_view',type='string')
                else:
                    obj._setProperty(id='layout',value='flickr_photo_view')
                obj.reindexObject()
                annotations = AttributeAnnotations(self.context)
            try:
                del annotations['slideshow_contents']
            except KeyError:
                pass
        return self.request.response.redirect('%s/view' % self.context.absolute_url())

