#!/usr/bin/python
# -*- coding: utf-8 -*-

import xmlrpclib

import urllib
import urllib2
from xml.dom.minidom import parseString

api_key = 'c41ba68569cb5eacc34e799ff54aa065'
server = 'http://www.flickr.com/services/rest/'


class BaseObject(object):
    '''
    >>> document= parseString('<obj id="test"><title>title</title></obj>')
    >>> node = document.getElementsByTagName('obj')[0]
    >>> obj = BaseObject(node)
    >>> obj.getId()
    u'test'
    >>> obj.Title()
    u'title'
    '''

    def __init__(self,node):
        self.node = node
        attributes = node.attributes
        for k,v in attributes.items():
            setattr(self,k,v)
        for n in node.childNodes:
            value = getattr(n.firstChild,'data','')
            value = value.replace('\n',' ')
            setattr(self,n.nodeName,value)

    def getId(self):
        return self.id
    
    def Title(self):
        return getattr(self,'title',self.id)

    def Description(self):
        return getattr(self,'description','')

class Sets(dict):
    ''' '''

class Set(BaseObject):
    ''' '''
    
    def __init__(self,node):
        BaseObject.__init__(self,node)
        self._objects = []
        client = FlickrClient()
        args = {'method':'flickr.photosets.getPhotos',
                'photoset_id':self.getId(),
                'extra':'license, date_upload, date_taken, owner_name, icon_server, original_format'}
        document = client.open(**args)
        nodes = document.getElementsByTagName('photo')
        for node in nodes:
            p = Photo(node)
            self._objects.append(p)

    def objectValues(self):
        return self._objects

    def __len__(self):
        return len(self._objects)

class SearchResults(Set):
    ''' '''
    
    def __init__(self,**args):
        self._objects = []
        client = FlickrClient()
        args['method'] = 'flickr.photos.search'
        document = client.open(**args)
        nodes = document.getElementsByTagName('photo')
        for node in nodes:
            p = Photo(node)
            self._objects.append(p)

class Photo(BaseObject):
    '''
    >>> document= parseString('<photo id="112132626"></photo>')
    >>> node = document.getElementsByTagName('photo')[0]
    >>> photo = Photo(node)
    >>> len(photo.Description()) > 0
    True
    '''

    def __init__(self,node):
        BaseObject.__init__(self,node)
        client = FlickrClient()
        args = {'method':'flickr.photos.getInfo',
                'photo_id':self.getId()}
        document = client.open(**args)
        nodes = document.getElementsByTagName('photo')
        BaseObject.__init__(self,nodes[0])
        nodes = document.getElementsByTagName('size')
        self._sizes = {}
        args = {'method':'flickr.photos.getSizes',
                'photo_id':self.getId()}
        document = client.open(**args)
        nodes = document.getElementsByTagName('size')
        for node in nodes:
            s = Size(node)
            self._sizes[s.getId()] = s
     
    def getAvailableSizes(self):
        return self._sizes.keys()
            
    def getWidth(self,scale='original'):
        return self._sizes[scale].width
    
    def getHeight(self,scale='original'):
        return self._sizes[scale].height
    
    def getURL(self,scale='original'):
        return self._sizes[scale].source
        base = 'http://static.flickr.com/%s/%s_%s' % (self.server,self.id,self.secret)
        return '%s_%s.jpg' % (base, scale[0])
    
class Size(BaseObject):
    ''' '''

    def __init__(self,node):
        BaseObject.__init__(self,node)
        self.id = self.label.lower()
        
class Client:
    '''
    >>> client = FlickrClient()
    >>> xml = client.open(method='flickr.test.echo')
    '''
    
    base = '%s?api_key=%s' % (server,api_key,)
    
    def __init__(self,):
        pass
        
    def open(self,**kwargs):
        args = ['%s=%s' % (k,v) for k,v in kwargs.items()]
        args = '&'.join(args)
        xml = urllib2.urlopen('%s&%s' % (self.base,args)).read()
        #print kwargs['method'], xml
        return parseString(xml)

def getUserId(username):
    """
    >>> getUserId('gawi')
    u'80142492@N00'
    >>> getUserId('gawiiiii')
    """
    client = FlickrClient()
    args = {'method':'flickr.people.findByUsername',
            'username':username}
    document = client.open(**args)
    nodes = document.getElementsByTagName('user')
    for node in nodes:
        user = BaseObject(node)
        return user.nsid
    return

def search(page=1,per_page=500,**args):
    """
    >>> r = search(tags='afpy')
    >>> len(r) > 1
    True
    """
    client = FlickrClient()
    results = SearchResults(page=page,
                            per_page=per_page,
                            **args)
    return results

def getSetsList(userid):
    '''
    >>> userid = getUserId('gawi')
    >>> sets = getSetsList(userid)
    >>> len(sets) > 0
    True
    '''
    sets = []
    args = {'method':'flickr.photosets.getList',
            'user_id':userid}
    client = FlickrClient()
    document = client.open(**args)
    nodes = document.getElementsByTagName('photoset')
    for node in nodes:
        s = BaseObject(node)
        sets.append(s)
    return sets

#def getSets(userid):
#    '''
#    >>> userid = getUserId('gawi')
#    >>> sets = getSets(userid)
#    >>> len(sets) > 0
#    True
#    >>> set = sets.items()[0][1]
#    >>> photo = set.objectValues()[0]
#    >>> len(photo.Title()) > 0
#    True
#    >>> photo.getAvailableSizes()
#    [u'small', u'large', u'medium', u'square', u'thumbnail']
#    '''
#    sets = Sets()
#    args = {'method':'flickr.photosets.getList',
#            'user_id':userid}
#    client = FlickrClient()
#    document = client.open(**args)
#    nodes = document.getElementsByTagName('photoset')
#    for node in nodes:
#        s = Set(node)
#        sets[s.getId()] = s
#    return sets

def getSet(node):
    """
    >>> userid = getUserId('gawi')
    >>> sets = getSetsList(userid)
    >>> set = sets[0]
    >>> set = getSet(set.node)
    >>> len(set) > 0
    True
    """
    return Set(node).objectValues()

global _FlickrClient
_FlickrClient = None

def FlickrClient():
    global _FlickrClient
    if not _FlickrClient:
        _FlickrClient = Client()
    return _FlickrClient

if __name__ == '__main__':
    import doctest
    doctest.testmod()
